@@ -8,7 +8,7 @@ As a concrete example, you can convert a newly downloaded Faroese WV file as fol
   python3 stanza/models/common/convert_pretrain.py ~/stanza/saved_models/pos/fo_farpahc.pretrain.pt ~/extern_data/wordvec/fasttext/faroese.txt -1
 or save part of an Icelandic WV file:
   python3 stanza/models/common/convert_pretrain.py ~/stanza/saved_models/pos/is_icepahc.pretrain.pt ~/extern_data/wordvec/fasttext/icelandic.cc.is.300.vec 150000
-Note that if the pretrain already exists, nothing will be changed.
+Note that if the pretrain already exists, nothing will be changed.  It will not overwrite an existing .pt file.
 """
 
 import os
@@ -19,7 +19,10 @@ from stanza.models.common import pretrain
 def main():
     filename = sys.argv[1]
     vec_filename = sys.argv[2]
-    max_vocab = int(sys.argv[3])
+    if len(sys.argv) < 3:
+        max_vocab = -1
+    else:
+        max_vocab = int(sys.argv[3])
 
     pt = pretrain.Pretrain(filename, vec_filename, max_vocab)
     print("Pretrain is of size {}".format(len(pt.vocab)))
