@@ -68,7 +68,7 @@ class SoftDotAttention(nn.Module):
         self.tanh = nn.Tanh()
         self.mask = None
 
-    def forward(self, input, context, mask=None, attn_only=False):
+    def forward(self, input, context, mask=None, attn_only=False, return_logattn=False):
         """Propagate input through the network.
 
         input: batch x dim
@@ -84,11 +84,16 @@ class SoftDotAttention(nn.Module):
             assert mask.size() == attn.size(), "Mask size must match the attention size!"
             attn.masked_fill_(mask, -constant.INFINITY_NUMBER)
 
-        attn = self.sm(attn)
+        if return_logattn:
+            attn = torch.log_softmax(attn, 1)
+            attn_w = torch.exp(attn)
+        else:
+            attn = self.sm(attn)
+            attn_w = attn
         if attn_only:
             return attn
 
-        attn3 = attn.view(attn.size(0), 1, attn.size(1))  # batch x 1 x sourceL
+        attn3 = attn_w.view(attn_w.size(0), 1, attn_w.size(1))  # batch x 1 x sourceL
 
         weighted_context = torch.bmm(attn3, context).squeeze(1)  # batch x dim
         h_tilde = torch.cat((weighted_context, input), 1)
@@ -210,22 +215,30 @@ class LSTMAttention(nn.Module):
             raise Exception("Unsupported LSTM attention type: {}".format(attn_type))
         logger.debug("Using {} attention for LSTM.".format(attn_type))
 
-    def forward(self, input, hidden, ctx, ctx_mask=None):
+    def forward(self, input, hidden, ctx, ctx_mask=None, return_logattn=False):
         """Propagate input through the network."""
         if self.batch_first:
             input = input.transpose(0,1)
 
         output = []
+        attn = []
         steps = range(input.size(0))
         for i in steps:
             hidden = self.lstm_cell(input[i], hidden)
             hy, cy = hidden
-            h_tilde, alpha = self.attention_layer(hy, ctx, mask=ctx_mask)
+            h_tilde, alpha = self.attention_layer(hy, ctx, mask=ctx_mask, return_logattn=return_logattn)
             output.append(h_tilde)
+            attn.append(alpha)
         output = torch.cat(output, 0).view(input.size(0), *output[0].size())
 
         if self.batch_first:
             output = output.transpose(0,1)
 
+        if return_logattn:
+            attn = torch.stack(attn, 0)
+            if self.batch_first:
+                attn = attn.transpose(0, 1)
+            return output, hidden, attn
+
         return output, hidden
 
