@@ -4,7 +4,7 @@ Common utilities for Stanza resources.
 
 import os
 import requests
-from tqdm import tqdm
+from tqdm.auto import tqdm
 from pathlib import Path
 import json
 import hashlib
@@ -24,7 +24,7 @@ logger = logging.getLogger('stanza')
 HOME_DIR = str(Path.home())
 STANFORDNLP_RESOURCES_URL = 'https://nlp.stanford.edu/software/stanza/stanza-resources/'
 STANZA_RESOURCES_GITHUB = 'https://raw.githubusercontent.com/stanfordnlp/stanza-resources/'
-DEFAULT_RESOURCES_URL = os.getenv('STANZA_RESOURCES_URL', STANZA_RESOURCES_GITHUB + 'master')
+DEFAULT_RESOURCES_URL = os.getenv('STANZA_RESOURCES_URL', STANZA_RESOURCES_GITHUB + 'main')
 DEFAULT_RESOURCES_VERSION = os.getenv(
     'STANZA_RESOURCES_VERSION',
     __resources_version__
@@ -103,12 +103,18 @@ def file_exists(path, md5):
     """
     return os.path.exists(path) and get_md5(path) == md5
 
-def download_file(url, path):
+def assert_file_exists(path, md5=None):
+    assert os.path.exists(path), "Could not find file at %s" % path
+    if md5:
+        file_md5 = get_md5(path)
+        assert file_md5 == md5, "md5 for %s is %s, expected %s" % (path, file_md5, md5)
+
+def download_file(url, path, proxies, raise_for_status=False):
     """
     Download a URL into a file as specified by `path`.
     """
     verbose = logger.level in [0, 10, 20]
-    r = requests.get(url, stream=True)
+    r = requests.get(url, stream=True, proxies=proxies)
     with open(path, 'wb') as f:
         file_size = int(r.headers.get('content-length'))
         default_chunk_size = 131072
@@ -120,8 +126,11 @@ def download_file(url, path):
                     f.write(chunk)
                     f.flush()
                     pbar.update(len(chunk))
+    if raise_for_status:
+        r.raise_for_status()
+    return r.status_code
 
-def request_file(url, path, md5=None):
+def request_file(url, path, proxies=None, md5=None, raise_for_status=False):
     """
     A complete wrapper over download_file() that also make sure the directory of
     `path` exists, and that a file matching the md5 value does not exist.
@@ -130,8 +139,8 @@ def request_file(url, path, md5=None):
     if file_exists(path, md5):
         logger.info(f'File exists: {path}.')
         return
-    download_file(url, path)
-    assert(not md5 or file_exists(path, md5))
+    download_file(url, path, proxies, raise_for_status)
+    assert_file_exists(path, md5)
 
 def sort_processors(processor_list):
     sorted_list = []
@@ -146,19 +155,6 @@ def maintain_processor_list(resources, lang, package, processors):
     # resolve processor models
     if processors:
         logger.debug(f'Processing parameter "processors"...')
-        # first, a hack!
-        # if tokenize is in the list, but mwt is not, and there is a corresponding
-        # tokenize & mwt pair in the resources file, we add mwt
-        # otherwise we'll get another 10 bugs regarding missing mwt errors
-        if TOKENIZE in processors and MWT not in processors:
-            value = processors[TOKENIZE]
-            if value == 'default' and MWT in resources[lang]['default_processors']:
-                logger.warning("Language %s package default expects mwt, which has been added" % lang)
-                processors[MWT] = 'default'
-            elif (value in resources[lang][TOKENIZE] and MWT in resources[lang] and
-                  value in resources[lang][MWT]):
-                logger.warning("Language %s package %s expects mwt, which has been added" % (lang, value))
-                processors[MWT] = value
         for key, value in processors.items():
             assert(isinstance(key, str) and isinstance(value, str))
             if key not in PIPELINE_NAMES:
@@ -330,8 +326,8 @@ def process_pipeline_parameters(lang, model_dir, package, processors):
 
     return lang, model_dir, package, processors
 
-def download_resources_json(model_dir, resources_url,
-                            resources_branch, resources_version):
+def download_resources_json(model_dir, resources_url, resources_branch,
+                            resources_version, proxies=None):
     """
     Downloads resources.json to obtain latest packages.
     """
@@ -344,19 +340,22 @@ def download_resources_json(model_dir, resources_url,
     # make request
     request_file(
         f'{resources_url}/resources_{resources_version}.json',
-        os.path.join(model_dir, 'resources.json')
+        os.path.join(model_dir, 'resources.json'),
+        proxies,
+        raise_for_status=True
     )
 
 
 def list_available_languages(model_dir=DEFAULT_MODEL_DIR,
                              resources_url=DEFAULT_RESOURCES_URL,
                              resources_branch=None,
-                             resources_version=DEFAULT_RESOURCES_VERSION):
+                             resources_version=DEFAULT_RESOURCES_VERSION,
+                             proxies=None):
     """
     List the non-alias languages in the resources file
     """
-    download_resources_json(model_dir, resources_url,
-                            resources_branch, resources_version)
+    download_resources_json(model_dir, resources_url, resources_branch,
+                            resources_version, proxies)
     with open(os.path.join(model_dir, 'resources.json')) as fin:
         resources = json.load(fin)
     # isinstance(str) is because of fields such as "url"
@@ -378,7 +377,8 @@ def download(
         resources_url=DEFAULT_RESOURCES_URL,
         resources_branch=None,
         resources_version=DEFAULT_RESOURCES_VERSION,
-        model_url=DEFAULT_MODEL_URL
+        model_url=DEFAULT_MODEL_URL,
+        proxies=None
     ):
     # set global logging level
     set_logging_level(logging_level, verbose)
@@ -387,8 +387,8 @@ def download(
         lang, model_dir, package, processors
     )
 
-    download_resources_json(model_dir, resources_url,
-                            resources_branch, resources_version)
+    download_resources_json(model_dir, resources_url, resources_branch,
+                            resources_version, proxies)
     # unpack results
     with open(os.path.join(model_dir, 'resources.json')) as fin:
         resources = json.load(fin)
@@ -408,7 +408,8 @@ def download(
         request_file(
             f'{url}/{resources_version}/{lang}/default.zip',
             os.path.join(model_dir, lang, f'default.zip'),
-            md5=resources[lang]['default_md5']
+            proxies,
+            md5=resources[lang]['default_md5'],
         )
         unzip(os.path.join(model_dir, lang), 'default.zip')
     # Customize: maintain download list
@@ -430,6 +431,7 @@ def download(
                 request_file(
                     f'{url}/{resources_version}/{lang}/{key}/{value}.pt',
                     os.path.join(model_dir, lang, key, f'{value}.pt'),
+                    proxies,
                     md5=resources[lang][key][value]['md5']
                 )
             except KeyError as e:
